#include "demo_types.h"
#include "demo_config.h"

uint8_t sensorResults[ 3 ];
uint8_t cnt;

void systemInit()
{
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    mikrobus_i2cInit( _MIKROBUS1, &_TOUCHKEY4_I2C_CFG[0] );
    mikrobus_logInit( _MIKROBUS2, 9600 );
    
    mikrobus_gpioInit( _MIKROBUS2, _MIKROBUS_PWM_PIN, _GPIO_OUTPUT );
    mikrobus_gpioInit( _MIKROBUS2, _MIKROBUS_RST_PIN, _GPIO_OUTPUT );
    mikrobus_gpioInit( _MIKROBUS2, _MIKROBUS_CS_PIN, _GPIO_OUTPUT );
    mikrobus_i2cInit( _MIKROBUS2, &_CLOCKGEN2_I2C_CFG[0] );
    Delay_ms( 100 );
}

void applicationInit()
{
    touchkey4_i2cDriverInit( (T_TOUCHKEY4_P)&_MIKROBUS1_GPIO, (T_TOUCHKEY4_P)&_MIKROBUS1_I2C, 0x28 );
    clockgen2_i2cDriverInit( (T_CLOCKGEN2_P)&_MIKROBUS2_GPIO, (T_CLOCKGEN2_P)&_MIKROBUS2_I2C, CLOCKGEN2_ADDR );
    mikrobus_logWrite( "TouchKey 4 is initialized", _LOG_LINE );
    Delay_ms( 200 );

    touchkey4_writeReg( _TOUCHKEY4_CONFIG2_REG, _TOUCHKEY4_AN_CALIB_RETRY | _TOUCHKEY4_DETECT_RELEASE_EN );
    touchkey4_writeReg( _TOUCHKEY4_REPEAT_RATE_EN_REG, _TOUCHKEY4_INPUT2_REPEAT_RATE_EN );
    touchkey4_writeReg( _TOUCHKEY4_CONFIG_REG, _TOUCHKEY4_AN_NOISE_FILTER_EN | _TOUCHKEY4_DIG_NOISE_THRESHOLD_DIS );
    touchkey4_writeReg( _TOUCHKEY4_STANDBY_THRESHOLD_REG, 0x40 );
    touchkey4_writeReg( _TOUCHKEY4_STANDBY_SENSITIVITY_REG, _TOUCHKEY4_STBY_SENSE_MULTIPLIER_64X );
    touchkey4_writeReg( _TOUCHKEY4_STANDBY_CONFIG_REG, _TOUCHKEY4_8_SAMPLES | _TOUCHKEY4_SAMPLE_TIME_1280MICROSEC | _TOUCHKEY4_CYCLE_TIME_70MILISEC );
    touchkey4_writeReg( _TOUCHKEY4_AVRG_AND_SAMPL_CONFIG_REG, _TOUCHKEY4_8_SAMPLES | _TOUCHKEY4_SAMPLE_TIME_1280MICROSEC | _TOUCHKEY4_CYCLE_TIME_70MILISEC );
    touchkey4_writeReg( _TOUCHKEY4_SENS_IN_CONFIG2_REG, _TOUCHKEY4_PRESS_AND_HOLD_EVENT_AFTER_280MILISEC );
    touchkey4_writeReg( _TOUCHKEY4_SENS_IN_CONFIG_REG, _TOUCHKEY4_5600MILISEC_BEFORE_RECALIB | _TOUCHKEY4_INTERR_REPEAT_RATE_175MILISEC );
    touchkey4_writeReg( _TOUCHKEY4_SENSITIVITY_CON_REG, _TOUCHKEY4_SENSITIVITY_MULTIPLIER_32X | _TOUCHKEY4_DATA_SCALING_FACTOR_256X );
    touchkey4_setComboMode( _TOUCHKEY4_GAIN_1, _TOUCHKEY4_COMBO_SENS_IN_ACT_EN, _TOUCHKEY4_COMBO_SENS_IN_BOTH_MODES_EN, _TOUCHKEY4_COMBO_SENS_IN_STBY_EN );
    Delay_ms( 200 );
}

void applicationTask()
{
    touchkey4_detectTouch( &sensorResults[0] );
    for (cnt = 0; cnt < 3; cnt++)
    {
        if (sensorResults[ cnt ] == 1)
        {
            if (cnt == 0)
            {
                mikrobus_logWrite( "Input 1 is touched", _LOG_LINE );
                clockgen2_outputEnable(0);
                clockgen2_setPrescaler(5);
                clockgen2_outputEnable(1);
            }
            else if (cnt == 1)
            {
                mikrobus_logWrite( "Input 2 is touched", _LOG_LINE );
                clockgen2_outputEnable(0);
                clockgen2_setPrescaler(6);
                clockgen2_outputEnable(1);
            }
            else
            {
                mikrobus_logWrite( "Input 3 is touched", _LOG_LINE );
                clockgen2_outputEnable(0);
                clockgen2_setPrescaler(7);
                clockgen2_outputEnable(1);
            }
        }
    }
}

void main()
{
    systemInit();
    applicationInit();

    while (1)
    {
        applicationTask();
    }
}
